# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Qt UI."""

import os
import subprocess
import sys

from PyQt4 import QtGui, QtCore
from ubuntuone.devtools.testcase import skipIfOS
from ubuntuone.platform.credentials import APP_NAME
from ubuntuone.controlpanel.gui import qt

from ubuntuone_installer.gui import NEXT
from ubuntuone_installer.gui.qt import gui, forgotten, utils
from ubuntuone_installer.gui.qt.tests import BaseTestCase, NO_OP


class FakeController(object):

    """A fake controller for the tests."""

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103
    is_fake = True


class FakeAreYouSure(object):

    """A fake Dialog."""

    result = 0
    shown = False

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    def exec_(self):
        """Fake showing the dialog."""
        self.shown = True
        return self.result  # Means reject


class FakeOverlay(object):

    """A fake delay overlay."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.show_counter = 0
        self.hide_counter = 0
        self.args = (args, kwargs)

    def show(self):
        """Fake show."""
        self.show_counter += 1

    def hide(self):
        """Fake hide."""
        self.hide_counter += 1

    def resize(self, *args):
        """Fake resize."""


class FakeLocalFoldersPage(QtGui.QWizardPage):
    """Fake Local Folders Page."""


class FakeUIFoldersPanel(QtGui.QWidget):
    """Fake UI Folders Page."""

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.share_publish_button = QtGui.QPushButton()
        self.add_folder_button = QtGui.QPushButton()
        self.folders = QtGui.QTreeWidget()
        self.folders.setColumnCount(3)
        self.folders.setColumnHidden(2, True)


class FakeFoldersPanel(QtGui.QWidget):
    """Fake Folders Page."""

    ui = FakeUIFoldersPanel()


class FakeUIPreferencesPanel(QtGui.QWidget):
    """Fake UI Preferences Page."""

    apply_changes_button = QtGui.QPushButton()
    restore_defaults_button = QtGui.QPushButton()


class FakePreferencesPanel(QtGui.QWidget):
    """Fake Preferences Page."""

    ui = FakeUIPreferencesPanel()
    apply_button_clicked = False
    restore_button_clicked = False

    def on_apply_changes_button_clicked(self):
        """Set apply_button_clicked to True for PreferencesPage."""
        self.apply_button_clicked = True

    def on_restore_defaults_button_clicked(self):
        """Set restore_button_clicked to True for PreferencesPage."""
        self.restore_button_clicked = True


class MainWindowTestCase(BaseTestCase):
    """Test the qt main window."""

    class_ui = gui.MainWindow

    def setUp(self):
        """Initialize this test instance."""
        # Faking each SSO object instead of doing it lower
        # so we don't rely on any SSO behaviour
        self.patch(gui, "SetUpAccountController", FakeController)
        self.patch(gui, "EmailVerificationController", FakeController)
        self.patch(gui, "CurrentUserController", FakeController)
        self.patch(gui, "SuccessController", FakeController)
        self.patch(gui, "ForgottenPasswordController", FakeController)
        self.patch(gui, "ResetPasswordController", FakeController)
        self.patch(gui, "AreYouSure", FakeAreYouSure)
        self.patch(gui, "LoadingOverlay", FakeOverlay)
        self.patch(gui, "LocalFoldersPage", FakeLocalFoldersPage)
        self.patch(qt.preferences, "PreferencesPanel", FakePreferencesPanel)
        self.patch(qt.folders, "FoldersPanel", FakeFoldersPanel)
        self.patch(gui.qt.utils, "add_syncdaemon_to_autostart", NO_OP)
        super(MainWindowTestCase, self).setUp()
        setup_page = self.ui.page(self.ui.setup_account_page_id)
        setup_page.initializePage()

    def test_wizard_init(self):
        """Check the Wizard flags regarding to the context help button."""
        flag = int(self.ui.windowFlags() &
            QtCore.Qt.WindowContextHelpButtonHint)
        self.assertEqual(flag, 0)

    def test_initialize_page(self):
        """Check the initializePage to ensure proper widgets visibility."""
        setup_page = self.ui.page(self.ui.setup_account_page_id)
        self.ui.setStartId(self.ui.setup_account_page_id)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertFalse(setup_page.ui.name_assistance.isVisible())
        self.assertFalse(setup_page.ui.email_assistance.isVisible())
        self.assertFalse(setup_page.ui.confirm_email_assistance.isVisible())
        self.assertFalse(setup_page.ui.password_assistance.isVisible())

    def test_setup_account_show_event_enabled_false(self):
        """Check the proper state of the Setup Account button on shown."""
        setup_page = self.ui.page(self.ui.setup_account_page_id)
        self.ui.setStartId(self.ui.setup_account_page_id)
        self.ui.restart()
        setup_page.set_up_button.setEnabled(False)
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(setup_page.set_up_button.isVisible())
        self.assertTrue(setup_page.set_up_button.isDefault())
        self.assertFalse(setup_page.set_up_button.isEnabled())
        self.assertTrue(setup_page.set_up_button.property("DisabledState"))

    def test_setup_account_hide_event_enabled_false(self):
        """Check the proper state of the Setup Account button on hide."""
        setup_page = self.ui.page(self.ui.setup_account_page_id)
        self.ui.setStartId(self.ui.setup_account_page_id)
        self.ui.restart()
        self.ui.show()
        self.ui.hide()
        self.addCleanup(self.ui.hide)
        self.assertFalse(setup_page.set_up_button.isVisible())
        self.assertTrue(setup_page.set_up_button.isDefault())

    def test_execute_uninstall_on_licence_cancel(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.ui.setStartId(self.ui.LICENSE_PAGE_ID)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        gui.AreYouSure.result = 0
        self.patch(utils, "uninstall_application", self._set_called)
        self.ui.done(result=0)
        self.assertTrue(self._called)

    def test_execute_uninstall_on_licence_cancel_frozen(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.ui.setStartId(self.ui.LICENSE_PAGE_ID)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        gui.AreYouSure.result = 0
        self.patch(utils, "uninstall_application", self._set_called)
        self.ui.done(result=0)
        self.assertTrue(self._called)

    def test_preferences_page(self):
        """Test Preferences Page UI display."""
        preferences_page = self.ui.page(self.ui.preferences_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.preferences_page_id)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)

        btn_pref = preferences_page.preferences_widget.ui.apply_changes_button
        self.assertEqual(btn_pref.isVisible(), False)
        btn_pref = \
            preferences_page.preferences_widget.ui.restore_defaults_button
        self.assertEqual(btn_pref.isVisible(), False)

    def test_preferences_page_navigation_buttons(self):
        """Check Preferences Page navigation buttons text."""
        preferences_page = self.ui.page(self.ui.preferences_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.preferences_page_id)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)

        btn_pref = preferences_page.wizard().button(
            QtGui.QWizard.CustomButton1)
        self.assertEqual(btn_pref.text(), "Apply these settings >")
        btn_pref = preferences_page.wizard().button(
            QtGui.QWizard.CustomButton2)
        self.assertEqual(btn_pref.text(), "Default settings")

    def test_preferences_page_button_clicked(self):
        """Check Preferences Page navigation buttons text."""
        preferences_page = self.ui.page(self.ui.preferences_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.preferences_page_id)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)

        # pylint: disable=W0212
        preferences_page._button_clicked(QtGui.QWizard.CustomButton2)
        self.assertEqual(
            preferences_page.preferences_widget.restore_button_clicked, True)
        preferences_page._button_clicked(QtGui.QWizard.CustomButton1)
        self.assertEqual(
            preferences_page.preferences_widget.apply_button_clicked, True)
        # for Cleanup
        btn_pref = preferences_page.wizard().button(QtGui.QWizard.NextButton)
        self.assertEqual(btn_pref.text(), NEXT)
        self.assertEqual(self.ui._next_id, self.ui.local_folders_page_id)

    def test_preferences_cleanup_page(self):
        """cleanupPage should call folders.initializePage()."""
        preferences_page = self.ui.page(self.ui.preferences_page_id)
        self.patch(self.ui.folders_page,
            "initializePage", self._set_called)
        preferences_page.cleanupPage()
        self.assertEqual(self._called, ((), {}))

    def test_folders_page_with_back(self):
        """Check Folders Page UI."""
        folders_page = self.ui.page(self.ui.folders_page_id)
        self.patch(self.ui, "setButtonLayout", self._set_called)

        # Show the preferences page
        self.ui.setStartId(self.ui.folders_page_id)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)

        btn_folders = folders_page.folders_widget.ui.share_publish_button
        self.assertEqual(btn_folders.isVisible(), False)
        btn_folders = folders_page.folders_widget.ui.add_folder_button
        self.assertEqual(btn_folders.isVisible(), False)
        tree_folders = folders_page.folders_widget.ui.folders
        self.assertEqual(tree_folders.isHeaderHidden(), True)
        tree_folders = folders_page.folders_widget.ui.folders
        self.assertEqual(tree_folders.isColumnHidden(2), True)
        # pylint: disable=W0212
        self.assertEqual(self.ui._next_id, self.ui.local_folders_page_id)
        self.assertEqual(self._called,
            (([QtGui.QWizard.Stretch,
             QtGui.QWizard.NextButton],), {}))

    def test_folders_page_next_id(self):
        """Check Folders Page UI."""
        folders_page = self.ui.page(self.ui.folders_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.folders_page_id)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)

        folders_page.on_check_settings_button_clicked()
        self.assertEqual(self.ui.currentId(), self.ui.preferences_page_id)

    def test_critical(self):
        """Show a critical popup, but first hide the overlay."""
        self.assertEqual(self.ui.overlay.hide_counter, 1)
        self.ui.critical("WORRY!")
        self.assertEqual(self.ui.overlay.hide_counter, 2)
        self.assertEqual(self.ui.form_errors_label.text(), "WORRY!")

    def test_critical_in_proper_page(self):
        """Check that the critical message is added in the proper page."""
        setup_page = self.ui.page(self.ui.setup_account_page_id)
        folders_page = self.ui.page(self.ui.folders_page_id)
        self.ui.critical("WORRY!", setup_page)
        self.assertEqual(self.ui.form_errors_label.text(), "WORRY!")
        self.assertEqual(self.ui.form_errors_label.parent(),
            setup_page)
        self.assertNotEqual(self.ui.form_errors_label.parent(),
            folders_page)
        self.ui.critical("WORRY!", folders_page)
        self.assertEqual(self.ui.form_errors_label.parent(),
            folders_page)
        self.assertNotEqual(self.ui.form_errors_label.parent(),
            setup_page)

    def test_forgotten_password_controller_error(self):
        """When there is an error, it should hide the overlay."""
        self.assertEqual(self.ui.overlay.hide_counter, 1)
        controller = forgotten.ForgottenPasswordController()
        controller.view = self.ui.forgotten
        controller.on_password_reset_error("app_name",
            {'errtype': 'ResetPasswordTokenError'})
        self.assertEqual(self.ui.overlay.hide_counter, 2)

    def test_overlay_connection_setup_account(self):
        """Make sure we don't lose connections to the overlay."""
        # Show the setup account page
        setup_page = self.ui.page(self.ui.setup_account_page_id)
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertEqual(self.ui.overlay.show_counter, 0)
        # This test is different from the ones below.
        # For some reason, click() doesn't trigger the connected
        # slot, but clicked.emit() does. The reason is unknown.
        setup_page.set_up_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_email_verification(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.email_verification.next_button.click()
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_current_user(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.current_user.ui.sign_in_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_forgotten_password(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.forgotten.ui.send_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_reset_password(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.reset_password.ui.reset_password_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_next_hides_overlay(self):
        """Make sure next() hides the overlay."""
        self.ui.setStartId(self.ui.LICENSE_PAGE_ID)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.next()
        # This is 2 because we hide it once on __init__()
        # and then again on next()
        self.assertEqual(self.ui.overlay.hide_counter, 2)

    def test_insert_remove_critical_message(self):
        """Make sure critical message is added and next() removes it."""
        self.ui.setStartId(self.ui.LICENSE_PAGE_ID)
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        license_page = self.ui.page(self.ui.LICENSE_PAGE_ID)
        self.assertEqual(self.ui.form_errors_label.parentWidget(), None)
        self.ui.critical("test critical message")
        self.assertEqual(self.ui.form_errors_label.parentWidget(),
            license_page)
        self.assertEqual(self.ui.form_errors_label.text(),
            "test critical message")
        self.ui.next()
        self.assertEqual(self.ui.form_errors_label.parentWidget(), None)

    def test_done_calls_custom_close_callback(self):
        """When closing the window, close_callback is called."""
        gui.AreYouSure.result = 1
        self.ui.close_callback = self._set_called
        self.ui.done(result=0)
        self.assertEqual(self._called,
                         ((), {}), 'close_callback called.')

    def test_close_callback_can_be_none(self):
        """The close_callback can be None."""
        gui.AreYouSure.result = 1
        self.ui.close_callback = None
        self.ui.done(result=0)
        # world did not explode

    def test_main_window_general_config(self):
        """Check if the main properties of the windows are set."""
        self.assertEqual(self.ui.windowTitle(), APP_NAME)

    def test_start_control_panel_on_finishing(self):
        """Calling done with result=1, the control panel should be called."""
        gui.AreYouSure.result = 1
        self.patch(subprocess, "Popen", self._set_called)
        self.ui.done(result=1)
        self.assertEqual(self._called,
            ((['ubuntuone-control-panel-qt'],), {}))

    @skipIfOS('linux2', 'Windows-specific tests.')
    def test_start_control_panel_frozen(self):
        """When frozen, the control-panel has a path."""
        self.ui.restart()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        gui.AreYouSure.result = 1
        # I can't patch sys because frozen is not there by default
        sys.frozen = True

        def clean_frozen():
            """Remove sys.frozen."""
            del sys.frozen

        self.addCleanup(clean_frozen)
        self.patch(subprocess, "Popen", self._set_called)
        self.ui.done(result=1)
        self.assertIn(os.sep, self._called[0][0][0])
        self.assertIn('ubuntuone-control-panel-qt', self._called[0][0][0])

    def test_not_start_control_panel_on_cancel(self):
        """Called done with result=0, control panel should NOT be called."""
        gui.AreYouSure.result = 1
        self.patch(subprocess, "Popen", self._set_called)
        self.ui.done(result=0)
        self.assertEqual(self._called, False)

    def test_continue_cancels_cancel(self):
        """Clicking 'Continue' cancels cancellation."""
        gui.AreYouSure.result = 1
        self.patch(subprocess, "Popen", self._set_called)
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.done(result=0)
        self.assertEqual(self.ui.isVisible(), True)

    def test_user_is_sure_in_last_page(self):
        """Cancelling on the last page is ok."""
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.patch(self.ui, 'currentId', lambda: self.ui.CONGRATULATIONS_PAGE)
        self.patch(subprocess, "Popen", self._set_called)
        self.ui.done(result=0)
        self.assertEqual(self._called, False)
        self.assertEqual(FakeAreYouSure.shown, False)

    def test_current_user_controller_parameters(self):
        """Compare controller parameters with expected values."""
        self.assertEqual(self.ui.current_user_controller.args,
        ((), {'message_box': self.ui,
              'title': gui.TITLE_STYLE % gui.SIGN_IN,
              'subtitle': gui.SIGN_IN_SUBTITLE}))

    def test_stage_progression_1(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.LICENSE_PAGE_ID)
        self.assertEqual(self.ui.sideWidget().stage, 0)

    def test_stage_progression_2(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SIGNIN_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_3(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.setup_account_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_4(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.email_verification_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_5(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.current_user_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_6(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SUCCESS_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_7(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.ERROR_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_8(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.forgotten_password_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_9(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.reset_password_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_10(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.local_folders_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 3)

    def test_stage_progression_11(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SYNC_NOW_OR_LATER_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 3)

    def test_stage_progression_12(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.CONGRATULATIONS_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 4)

    def test_choose_signin_cancel(self):
        """Test that the custom cancel button works."""
        self.ui.sign_in_page.ui.cancel_button.click()
        QtCore.QCoreApplication.instance().processEvents()
        self.assertEqual(self.ui.result(), self.ui.Rejected)

    def test_congratulations_adds_to_autostart(self):
        """Test that syncdaemon is now on autostart."""
        self.patch(gui.qt.utils, "add_syncdaemon_to_autostart",
            self._set_called)
        # Show the Congratulations page
        self.ui.setStartId(self.ui.CONGRATULATIONS_PAGE)
        self.ui.restart()
        # Check if syncdaemon was added to autostart
        self.assertEqual(self._called, ((), {}))


class FakeSignal(object):

    """A fake PyQt signal."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.target = None

    def connect(self, target):
        """Fake connect."""
        self.target = target

    def disconnect(self, *args):
        """Fake disconnect."""
        self.target = None

    def emit(self, *args):
        """Fake emit."""
        if self.target:
            self.target(*args)


class FakeWizardPage(object):

    """A fake wizard page."""

    def __init__(self, *args, **kwargs):
        self.has_back_button = True

    # pylint: disable=C0103
    def initializePage(self):
        """Fake initializePage."""


class FakeMainWindow(object):

    """A fake MainWindow."""

    currentIdChanged = FakeSignal()
    loginSuccess = FakeSignal()
    registrationSuccess = FakeSignal()
    userCancellation = FakeSignal()
    shown = False
    SYNC_NOW_OR_LATER_PAGE = 4
    CONGRATULATIONS_PAGE = 5
    folders_page_id = 6
    local_folders_page_id = 7

    def __init__(self, close_callback=None):
        self.button_texts = []
        self.button_layout = None
        self.options = []
        self.overlay = FakeOverlay()
        self.local_folders_page = FakeWizardPage()
        self.folders_page = FakeWizardPage()

    def show(self):
        """Fake method."""
        self.shown = True

    # pylint: disable=C0103
    def setButtonText(self, button, text):
        """Fake setButtonText."""
        self.button_texts.append((button, text))

    def setButtonLayout(self, layout):
        """Fake setButtonText."""
        self.button_layout = layout

    def setOption(self, *args):
        """Fake setOption."""
        self.options.append(args)

    def next(self):
        """Fake next."""


class SuccesPageTestCase(BaseTestCase):
    """Test the SuccessPage code."""

    class_ui = gui.SuccessPage

    def test_after_initialize(self):
        """Check the page is initialized correctly."""
        wizard = FakeMainWindow()
        self.patch(self.ui, 'wizard', lambda *args: wizard)
        self.ui.initializePage()
        expected = sorted([
            (QtGui.QWizard.FinishButton, gui.SKIP_TOUR),
            (QtGui.QWizard.NextButton, gui.START_SETUP),
            ])
        produced = sorted(wizard.button_texts)
        self.assertEqual(expected, produced)
        self.assertTrue(self.ui.isFinalPage())
        self.assertEqual(wizard.button_layout,
        [QtGui.QWizard.Stretch,
         QtGui.QWizard.FinishButton,
         QtGui.QWizard.NextButton])
