# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#          Diego Sarmentero <diego.sarmentero@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Setup Account page Qt UI."""

from PyQt4 import QtGui, QtCore

from ubuntu_sso.qt import common

from ubuntuone_installer.gui.qt import gui, setup_account
from ubuntuone_installer.gui.qt.tests import BaseTestCase
from ubuntuone_installer.gui.qt.ui import (
    setup_account_ui,
)


class FakeController(object):

    """A fake controller for the tests."""

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103


class FakeWizard(object):
    """Replace wizard() function on wizard pages."""

    customButtonClicked = QtCore.QObject()
    overlay = None
    params = None

    # Invalid name "setButtonLayout"
    # pylint: disable=C0103

    def setButtonLayout(self, *args, **kwargs):
        """Fake the functionality of setButtonLayout on QWizard class."""
        FakeWizard.params = (args, kwargs)

    # pylint: enable=C0103

    # Invalid name "setOption"
    # pylint: disable=C0103

    def setOption(self, *args, **kwargs):
        """Fake the functionality of setOption on QWizard class."""
        FakeWizard.params = (args, kwargs)

    # pylint: enable=C0103

    def button(self, *args, **kwargs):
        """Fake the functionality of button on QWizard class."""
        return QtGui.QPushButton()


class FakeValidationDict(object):
    """A Fake dictionary to replace validation_functions on Setup Account."""

    def __init__(self):
        for i in xrange(len(self)):
            # set all the instance variables
            setattr(FakeValidationDict, 'valid%s' % i, False)
            setattr(self, 'validate_%s' % i, self.wrapper(i))

    def __getitem__(self, i):
        """Get the proper function depending on the index received."""
        return getattr(self, 'validate_%s' % i)

    def __getslice__(self, i, j):
        """Check that the class length is not exceded and return the funcs."""
        j = j if j <= len(self) else len(self)
        return [self[q] for q in xrange(j)]

    def __len__(self):
        """Return the number of functions in this class."""
        return 4

    def wrapper(self, i):
        """Change the state of the attributes based on the index."""
        def inner(*args, **kwargs):
            """Replace the value of the attributes depending on the index."""
            for j in xrange(i + 1):
                setattr(FakeValidationDict, 'valid%s' % j, True)
        return inner


class SetupAccountTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    class_ui = setup_account.SetupAccountPage
    kwargs = dict(
        ui=setup_account_ui.Ui_SetUpAccountPage(),
        controller=FakeController(),
        parent=None,
    )

    def setUp(self):
        super(SetupAccountTestCase, self).setUp()
        self.patch(self.ui, "set_up_button", QtGui.QPushButton())

    def test_init(self):
        """Check the initial state of SetupAccount."""
        self.assertEqual(self.ui.ui.password_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 1)
        self.assertTrue(self.ui.validation_functions)

    def test_empty_name(self):
        """Status when the name field is empty.

        * Name Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.name_edit.setText("")
        self.ui.name_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(self.ui.ui.name_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            setup_account.ERROR % setup_account.EMPTY_NAME)
        self.ui.hide()

    def test_password_default_assistance(self):
        """Status when the password line edit receive focus and shows popup.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("")
        self.ui.ui.confirm_password_edit.setText("")
        self.ui.validate_data_on_focus_changed(4)
        self.ui.focus_changed(None, self.ui.ui.password_edit)
        self.assertIn(
            common.NORMAL % common.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            common.NORMAL % common.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            common.NORMAL % common.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_side_widget_state(self):
        """Test if the side widget of the wizard has state frame visible."""
        if type(self.ui) != gui.MainWindow:
            return
        self.ui.on_currentIdChanged(self.ui.LICENSE_PAGE)
        self.assertTrue(self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.SIGNIN_PAGE)
        self.assertTrue(self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.local_folders_page_id)
        self.assertTrue(self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.CONGRATULATIONS_PAGE)
        self.assertTrue(self.ui.sideWidget().ui.states_frame.isVisible())

    def test_focus_changed_1(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(self.ui, 'validation_functions', FakeValidationDict())
        self.ui.validate_data_on_focus_changed(1)
        self.assertTrue(getattr(FakeValidationDict, "valid0"))
        self.assertFalse(getattr(FakeValidationDict, "valid1"))
        self.assertFalse(getattr(FakeValidationDict, "valid2"))
        self.assertFalse(getattr(FakeValidationDict, "valid3"))

    def test_focus_changed_2(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(self.ui, 'validation_functions', FakeValidationDict())
        self.ui.validate_data_on_focus_changed(2)
        self.assertTrue(getattr(FakeValidationDict, "valid0"))
        self.assertTrue(getattr(FakeValidationDict, "valid1"))
        self.assertFalse(getattr(FakeValidationDict, "valid2"))
        self.assertFalse(getattr(FakeValidationDict, "valid3"))

    def test_focus_changed_3(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(self.ui, 'validation_functions', FakeValidationDict())
        self.ui.validate_data_on_focus_changed(3)
        self.assertTrue(getattr(FakeValidationDict, "valid0"))
        self.assertTrue(getattr(FakeValidationDict, "valid1"))
        self.assertTrue(getattr(FakeValidationDict, "valid2"))
        self.assertFalse(getattr(FakeValidationDict, "valid3"))

    def test_focus_changed_4(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(self.ui, 'validation_functions', FakeValidationDict())
        self.ui.validate_data_on_focus_changed(4)
        self.assertTrue(getattr(FakeValidationDict, "valid0"))
        self.assertTrue(getattr(FakeValidationDict, "valid1"))
        self.assertTrue(getattr(FakeValidationDict, "valid2"))
        self.assertTrue(getattr(FakeValidationDict, "valid3"))


class SetupAccountFakeWizardTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    class_ui = setup_account.SetupAccountPage
    kwargs = dict(
        ui=setup_account_ui.Ui_SetUpAccountPage(),
        controller=FakeController(),
        parent=None,
    )

    def setUp(self):
        """Initialize this test instance."""
        # Faking each SSO object instead of doing it lower
        # so we don't rely on any SSO behaviour
        super(SetupAccountFakeWizardTestCase, self).setUp()
        self.patch(self.ui, 'wizard', FakeWizard)

    def test_blank_name(self):
        """Status when the name field is blank (spaces).

        * Name Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.name_edit.setText("  ")
        self.ui.name_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(self.ui.ui.name_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            setup_account.ERROR % setup_account.EMPTY_NAME)
        self.ui.hide()

    def test_valid_name(self):
        """Status when the name field is valid.

        * Name Assistance label invisible.
        """
        self.ui.ui.name_edit.setText("John Doe")
        self.ui.name_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertFalse(self.ui.ui.name_assistance.isVisible())
        self.ui.hide()

    def test_invalid_email(self):
        """Status when the email field has no @.

        * Email Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.email_edit.setText("foobar")
        self.ui.email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(self.ui.ui.email_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.email_assistance.text()),
            setup_account.ERROR % setup_account.INVALID_EMAIL)

    def test_valid_email(self):
        """Status when the email field has a @.

        * Email Assistance label invisible.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertFalse(self.ui.ui.email_assistance.isVisible())
        self.ui.hide()

    def test_matching_emails(self):
        """Status when the email fields match.

        * Email Assistance label invisible.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@bar")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertFalse(self.ui.ui.confirm_email_assistance.isVisible())
        self.ui.hide()

    def test_not_matching_emails(self):
        """Status when the email fields don't match.

        * Email Assistance label visible.
        * Correct error message.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@baz")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(self.ui.ui.confirm_email_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.confirm_email_assistance.text()),
            setup_account.ERROR % setup_account.EMAIL_MATCH)
        self.ui.hide()

    def test_focus_changed_password_visibility(self):
        """Check visibility changes when focus_changed() is executed."""
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.focus_changed(None, self.ui.ui.password_edit)
        self.assertTrue(self.ui.ui.password_assistance.isVisible())

    def test_hide_event(self):
        """Check that hide_event works when set_up_button is none."""
        self.assertEqual(self.ui.set_up_button, None)
        self.ui.hideEvent(QtGui.QHideEvent())

    def test_show_event(self):
        """Check that show_event works when set_up_button is none."""
        self.assertEqual(self.ui.set_up_button, None)
        self.ui.showEvent(QtGui.QShowEvent())
