# -*- coding: utf-8 -*-

# Authors: Diego Sarmentero <diego.sarmentero@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the platform specific functions."""

import sys
import os

# Avoid pylint error on Linux
# pylint: disable=F0401
import win32api
# pylint: enable=F0401

from ubuntuone_installer.gui.qt import utils
from ubuntuone_installer.gui.qt.tests import BaseTestCase


class UninstallerTestCase(BaseTestCase):
    """Test the uninstaller."""

    def test_execute_uninstall_on_licence_cancel_file_not_exists(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        # If it's frozen this test fails because the path
        # depends on the location of the exe
        frozen = getattr(sys, "frozen", None)
        if frozen is not None:
            delattr(sys, "frozen")
            self.addCleanup(setattr, sys, 'frozen', frozen)
        self.patch(win32api, "ShellExecute", self._set_called)
        self.assertFalse(self._called)

    def test_execute_uninstall_on_licence_cancel_frozen_file_not_exists(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.patch(win32api, "ShellExecute", self._set_called)
        sys.frozen = True
        self.addCleanup(delattr, sys, "frozen")
        utils.uninstall_application()
        self.assertFalse(self._called)

    def test_execute_uninstall_on_licence_cancel(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.patch(win32api, "ShellExecute", self._set_called)
        self.patch(os.path, "exists", lambda path: True)
        utils.uninstall_application()
        # If it's frozen this test fails because the path
        # depends on the location of the exe
        frozen = getattr(sys, "frozen", None)
        if frozen is not None:
            delattr(sys, "frozen")
            self.addCleanup(setattr, sys, 'frozen', frozen)
        exec_path = os.path.abspath(__file__)
        folder_path_utils_dir = os.path.dirname(exec_path)
        folder_path_qt_dir = os.path.dirname(folder_path_utils_dir)
        folder = os.path.dirname(folder_path_qt_dir)
        uninstall_path = os.path.join(os.path.dirname(folder), "uninstall.exe")
        self.assertEqual(self._called,
            ((None, 'runas', uninstall_path, '--mode qt', '', 0), {}))

    def test_execute_uninstall_on_licence_cancel_frozen(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.patch(win32api, "ShellExecute", self._set_called)
        self.patch(os.path, "exists", lambda path: True)
        sys.frozen = True
        self.addCleanup(delattr, sys, "frozen")
        utils.uninstall_application()
        self.assertTrue(self._called[0][0] is None)
        self.assertTrue("uninstall.exe" in self._called[0][2])
        self.assertTrue('runas' == self._called[0][1])
        self.assertTrue('--mode qt' == self._called[0][3])
        self.assertTrue('' == self._called[0][4])
        self.assertTrue(0 == self._called[0][5])


class FakeRegistry(object):
    """A fake registry."""
    # pylint: disable=C0103
    HKEY_CURRENT_USER = 2
    KEY_ALL_ACCESS = 4
    REG_SZ = 1

    def __init__(self):
        self.HKEY_CURRENT_USER = 2
        self.KEY_ALL_ACCESS = 4
        self.connect_args = None
        self.openkey_args = None
        self.closekey_args = None
        self.query_args = None
        self.set_args = []

    def ConnectRegistry(self, *args, **kwargs):
        """Fake ConnectRegistry."""
        self.connect_args = (args, kwargs)
        return "REGISTRY"

    def OpenKey(self, *args, **kwargs):
        """Fake OpenKey."""
        self.openkey_args = (args, kwargs)
        return "KEY"

    def CloseKey(self, *args, **kwargs):
        """Fake CloseKey."""
        self.closekey_args = (args, kwargs)

    def QueryValueEx(self, *args, **kwargs):
        """Fake QueryValueEx."""
        self.query_args = (args, kwargs)

    def SetValueEx(self, *args, **kwargs):
        """Fake SetValueEx."""
        self.set_args.append((args, kwargs))


class AutostartTestCase(BaseTestCase):
    """Test add_syncdaemon_to_autostart."""

    def setUp(self):
        """Initialize this test instance."""
        self.registry = FakeRegistry()
        self.patch(utils.windows, "_winreg", self.registry)
        super(AutostartTestCase, self).setUp()

    def test_add_syncdaemon_to_autostart(self):
        """Check that the registry is updated correctly."""
        # I can't patch sys because frozen is not there by default
        sys.frozen = True

        def clean_frozen():
            """Remove sys.frozen."""
            del sys.frozen

        self.addCleanup(clean_frozen)
        utils.windows.add_syncdaemon_to_autostart()
        self.assertEqual(self.registry.connect_args,
            ((None, self.registry.HKEY_CURRENT_USER), {}))
        self.assertEqual(self.registry.openkey_args,
            (('REGISTRY', utils.windows.AUTORUN_KEY, 0,
              self.registry.KEY_ALL_ACCESS), {}))
        self.assertEqual(self.registry.closekey_args, (('KEY',), {}))
        self.assertEqual(self.registry.query_args, None)
        self.assertEqual(self.registry.set_args,
             [(('KEY', 'Ubuntu One', 0, 1,
             'C:\\Python27\\ubuntuone-syncdaemon.exe'), {}),
             (('KEY',  'Ubuntu One Icon', 0, 1,
             'C:\\Python27\\ubuntuone-control-panel-qt.exe --minimized'),
             {})])

    def test_not_added_if_not_frozen(self):
        """Not frozen binaries are not added to the registry."""
        utils.windows.add_syncdaemon_to_autostart()
        self.assertEqual(self.registry.connect_args, None)
        self.assertEqual(self.registry.openkey_args, None)
        self.assertEqual(self.registry.closekey_args, None)
        self.assertEqual(self.registry.query_args, None)
        self.assertEqual(self.registry.set_args, [])


class DefaultFoldersTestCase(BaseTestCase):

    """Test the default suggested UDFs."""

    def test_special_folders(self):
        """Test that default_folders does the right thing."""
        folders = utils.default_folders()
        self.assertEqual(len(folders), 3)
        self.assertEqual(folders, [os.path.abspath(f) for f in folders])
        # pylint: disable=W0404
        import ctypes
        dll = ctypes.windll.shell32
        buf = ctypes.create_string_buffer(300)
        dll.SHGetSpecialFolderPathA(None, buf, 5, False)
        docs = buf.value
        dll.SHGetSpecialFolderPathA(None, buf, 13, False)
        music = buf.value
        dll.SHGetSpecialFolderPathA(None, buf, 39, False)
        pictures = buf.value
        expected = [docs, music, pictures]
        self.assertEqual(sorted(folders), sorted(expected))
